/******************************************************************************
 *            g_fcts_annexes.c                                                *
 *                                                                            *
 *  Copyright  2006  gerald.dumas                                             *
 *  linechec@laposte.net                                                      *
 ******************************************************************************
 *                                                                            *
 *  This program is free software; you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation; either version 2 of the License, or         *
 *  any later version.                                                        *
 *                                                                            *
 *  This program is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with this program; if not, write to the Free Software               *
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
 ******************************************************************************/

#include "g_fcts_annexes.h"

GtkWidget *
gtk_image_resize (GtkWidget * image, gint width, gint height)
{
  GtkWidget *image_dest;
  GdkPixbuf *src;
  GdkPixbuf *dest;

  src = gtk_image_get_pixbuf (GTK_IMAGE (image));

  //Redimensionnement de l'image et résultat dans dest
  dest = gdk_pixbuf_scale_simple (src, width, height, GDK_INTERP_BILINEAR);

  //Conversion du gdkpixbuf dest en un GtkWidget utilisable
  image_dest = gtk_image_new_from_pixbuf (dest);
  // DEBUG by xcfaudio: Test if the pointer is valid
  if( NULL != src )
    g_object_unref (src);
  // DEBUG by xcfaudio: Test if the pointer is valid
  if( NULL != dest )
    g_object_unref (dest);

  return image_dest;
}

GtkWidget *
gtk_image_new_from_file_with_size (gchar * filename, gint width, gint height)
{
  GdkPixbuf *dest;
  GtkWidget *image;

  dest =
    gdk_pixbuf_new_from_file_at_scale (filename, width, height, FALSE, NULL);

  //Conversion du gdkpixbuf dest en GtkWidget utilisable
  if (dest)
    image = gtk_image_new_from_pixbuf (dest);
  else
    image =
      gtk_image_new_from_stock (GTK_STOCK_MISSING_IMAGE,
				GTK_ICON_SIZE_DIALOG);

  // DEBUG by xcfaudio: Test if the pointer is valid
  if( NULL != dest )
    g_object_unref (dest);

  return image;
}

GtkWidget *
fen_dialog (GtkWidget * fenetre, gchar * titre_texte, ...)
{
  GtkWidget *image_soft;
  gchar *argument;
  GtkWidget *label_titre;
  GtkWidget *label_texte;
  GtkWidget *ligne_sep_hor;
  va_list arg_ptr;
  gchar *chemin = NULL;
  gchar *tmp;
	GdkPixbuf *icone_appli;
	icone_appli=(GdkPixbuf*)
								(gtk_data_list_get_by_name(widget_list, "icone_appli"));

  // Initialisation du pointeur vers les arguments variables
  va_start (arg_ptr, titre_texte);

  // affectation d'une icône à la fenêtre
  gtk_window_set_icon (GTK_WINDOW (fenetre), icone_appli);

  // Insertion de l'image du soft en tête de la fenêtre
	tmp=g_key_file_get_string(key_file_config, "DIRECTORY", "install", NULL);
	if (strcmp(tmp, "/usr/bin")==0)
	{
		g_free(tmp);
		tmp=g_strdup("/usr/share/linechec/pixmaps");
	}
	else
	{
		g_free(tmp);
		tmp=g_strdup_printf("%s/pixmaps",g_key_file_get_string(key_file_config,
																							"DIRECTORY", "install", NULL));
	}
	chemin = g_strdup_printf ("%s/linechec_bandeau.jpg", tmp);
  g_free (tmp);
  image_soft = gtk_image_new_from_file (chemin);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fenetre)->vbox), image_soft, TRUE,
		      TRUE, 0);

  // Insertion d'une ligne séparatrice horizontale
  ligne_sep_hor = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fenetre)->vbox), ligne_sep_hor,
		      TRUE, TRUE, 4);

  // Insertion du titre du texte
  label_titre = gtk_label_new (titre_texte);
  gtk_label_set_justify (GTK_LABEL (label_titre), GTK_JUSTIFY_CENTER);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fenetre)->vbox), label_titre, TRUE,
		      TRUE, 0);
  // Insertion d'un saut de ligne
  label_titre = gtk_label_new ("");

  // Tant que l'argument n'est pas = NULL ( dernier argument ))
  while ((argument = va_arg (arg_ptr, gchar *)) != NULL)
    {
      label_texte = gtk_label_new (argument);
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fenetre)->vbox), label_texte,
			  TRUE, TRUE, 0);
      gtk_label_set_justify (GTK_LABEL (label_titre), GTK_JUSTIFY_LEFT);
    }

  // Fermeture et nettoyage des != pointeurs utilisés
  va_end (arg_ptr);
  g_free (chemin);
  return fenetre;
}

void
insertion_texte_fen_etat_connexion (const gchar * buffer)
{
	GtkAdjustment	*ajustment;
	GtkWidget	*view=(GtkWidget*) (gtk_data_list_get_by_name(widget_list, "view"));
	
	// DEBUG by xcfaudio: Test if the pointer is valid
	if( NULL != buffer ) {
		if (!g_utf8_validate (buffer, -1, NULL))
			gtk_text_buffer_insert_at_cursor( gtk_text_view_get_buffer( GTK_TEXT_VIEW(view)), g_locale_to_utf8( buffer, -1, NULL, NULL, NULL), -1 );
		else
			gtk_text_buffer_insert_at_cursor( gtk_text_view_get_buffer( GTK_TEXT_VIEW(view)), buffer, -1 );

		ajustment = gtk_scrolled_window_get_vadjustment( GTK_SCROLLED_WINDOW(gtk_widget_get_ancestor(view, GTK_TYPE_SCROLLED_WINDOW)));
		ajustment->value = ajustment->upper;
		gtk_scrolled_window_set_vadjustment (GTK_SCROLLED_WINDOW(gtk_widget_get_ancestor(view, GTK_TYPE_SCROLLED_WINDOW)),ajustment);
		while (gtk_events_pending ())
			gtk_main_iteration ();
	}
}

GSList *gtk_data_list_add_by_name(GSList *list, gpointer data, gchar *name)
{
	/* Déclaration d'une structure interne pour mémoriser le pointeur du widget
	ainsi que son nom.*/
	struct Struct_list_widget
	{
		gpointer Sdata;
		gchar *Sname;
	};
	struct Struct_list_widget *Slist_widget=NULL;

	// Allocation mémoire d'une nouvelle structure. Le transtypage n'est
	// nécessaire que si vous compilez en C++.
	Slist_widget=(struct Struct_list_widget*)
		g_malloc(sizeof(struct Struct_list_widget));
	// Affection des valeurs
	Slist_widget->Sdata=data;
	Slist_widget->Sname=g_strdup(name);

	// Ajout des nouvelles données à la liste.
	list=g_slist_append(list, Slist_widget);

	return list;
}

GSList *gtk_data_list_remove_by_name(GSList *list, gchar *name)
{
	GSList *tmp_list=NULL;
	/* Déclaration d'une structure interne pour mémoriser le pointeur du widget
	ainsi que son nom.*/
	struct Struct_list_widget
	{
		gpointer Sdata;
		gchar *Sname;
	};

	// Recherche de l'objet dans la liste
	tmp_list=list;
	while (tmp_list)
	{
		if (strcmp(((struct Struct_list_widget*)(tmp_list->data))->Sname,name)==0)
		{
			// Supression de l'objet
			g_free(((struct Struct_list_widget*)(tmp_list->data))->Sname);
			g_free((struct Struct_list_widget*)(tmp_list->data));
			list=g_slist_remove_link(list, tmp_list);
			g_slist_free_1(tmp_list);
			return list;
		}
		tmp_list=g_slist_next(tmp_list);
	}
	
	return NULL;
}

gpointer *gtk_data_list_get_by_name(GSList *list, gchar *name)
{
	GSList *tmp_list=NULL;
	/* Déclaration d'une structure interne pour mémoriser le pointeur du widget
	ainsi que son nom.*/
	struct Struct_list_widget
	{
		gpointer Sdata;
		gchar *Sname;
	};

	// Recherche de l'objet dans la liste
	tmp_list=list;
	while (tmp_list)
	{
		if (strcmp(((struct Struct_list_widget*)(tmp_list->data))->Sname,name)==0)
			return ((struct Struct_list_widget*)(tmp_list->data))->Sdata;
		tmp_list=g_slist_next(tmp_list);
	}

	return NULL;
}

void gtk_data_list_remove_all(GSList *list)
{
	GSList *tmp_list=list;
	/* Déclaration d'une structure interne pour mémoriser le pointeur du widget
	ainsi que son nom.*/
	struct Struct_list_widget
	{
		gpointer Sdata;
		gchar *Sname;
	};

	while (tmp_list)
	{
		// Supression de l'objet
		g_free(((struct Struct_list_widget*)(tmp_list->data))->Sname);
		g_free((struct Struct_list_widget*)(tmp_list->data));
		tmp_list=g_slist_next(tmp_list);
	}
	g_slist_free(list);
}
